//+FHDR--------------------------------------------------------------------------------------------------------
//  Company: 
//-----------------------------------------------------------------------------------------------------------------
//  File Name             :    IIR_Filter_p1.v
//  Department            :    
//  Author                :    hdzhang
//  Author's Tel          :     
//-----------------------------------------------------------------------------------------------------------------
//  Relese History
//  Version     Date            Author          Description
//  0.0         2025-03-09      hdzhang
//2024-05-28 10:22:49 
//-----------------------------------------------------------------------------------------------------------------
//  Keywords            :       
//
//-----------------------------------------------------------------------------------------------------------------
//  Parameter
//
//-----------------------------------------------------------------------------------------------------------------
//  Purpose                 :
//                      
//-----------------------------------------------------------------------------------------------------------------
//  Target Device:        
//  Tool versions:        
//-----------------------------------------------------------------------------------------------------------------
//  Reuse Issues
//  Reset Strategy: 
//  Clock Domains: 
//  Critical Timing:
//  Asynchronous I/F:
//  Synthesizable (y/n): 
//  Other:
//-FHDR--------------------------------------------------------------------------------------------------------
module  IIR_Filter_p1 #(
 parameter  data_in_width       = 16 
,parameter  coef_width          = 32 
,parameter  frac_data_out_width = 20//X for in,5
,parameter  frac_coef_width     = 31//division
)
//H(z) = a / (1 - b*z^-1)
(
 input   rstn
,input   clk
,input   en
,input   signed [data_in_width-1:0]   din_re     // Re(x(t))
//,input   signed       [data_in_width-1:0]   din_im     // Im(x(t))
,input   signed [data_in_width-1:0]   dout_r1_re // Re(y(t-1))
,input   signed [data_in_width-1:0]   dout_r1_im // Im(y(t-1))
,input   signed [coef_width-1   :0]   a_re
,input   signed [coef_width-1   :0]   a_im
,input   signed [coef_width-1   :0]   b_re
,input   signed [coef_width-1   :0]   b_im

,output  signed [data_in_width-1:0]   dout_re // Re(y(t-16))
,output  signed [data_in_width-1:0]   dout_im // Im(y(t-16))
);


wire    signed  [data_in_width+frac_data_out_width-1:0] x1_re;
wire    signed  [data_in_width+frac_data_out_width-1:0] x1_im;

wire    signed  [data_in_width+frac_data_out_width-1:0] y1_re;
wire    signed  [data_in_width+frac_data_out_width-1:0] y1_im;
wire    signed  [data_in_width+frac_data_out_width  :0] y_re;
wire    signed  [data_in_width+frac_data_out_width  :0] y_im;

wire    signed  [data_in_width-1:0]   y_re_trunc;
wire    signed  [data_in_width-1:0]   y_im_trunc;


// x1 = a * din delay M = a*x(t-8)
mult_x
#(
	 .A_width		(data_in_width			)  
	,.C_width		(coef_width+frac_data_out_width	) 
	,.D_width		(coef_width+frac_data_out_width	) 
	,.frac_coef_width	(frac_coef_width		) 
)
inst_c1 (
	.clk        	(clk        				),
	.rstn       	(rstn           			),
	.en         	(en             			),
	.a          	(din_re         			),
	.c          	({a_re,{frac_data_out_width{1'b0}}}	),
	.d          	({a_im,{frac_data_out_width{1'b0}}}	),
	.Re         	(x1_re          			),
	.Im         	(x1_im          			)
);


// y1 = b * dout_r1 delay M = b*y(t-9)
// y = y1+x1 = a*x(t-8)+b*y(t-9) = y(t-8) 
mult_C
#(
 .A_width(data_in_width) 
,.B_width(data_in_width) 
,.C_width(coef_width+frac_data_out_width) 
,.D_width(coef_width+frac_data_out_width) 
,.frac_coef_width(frac_coef_width) 
)
inst_c3 (
	.clk        (clk                  		),
	.rstn       (rstn                 		),
	.en         (en                   		),
 	.a          (dout_r1_re             		),
	.b          (dout_r1_im             		),
	.c          ({b_re,{frac_data_out_width{1'b0}}}	),
	.d          ({b_im,{frac_data_out_width{1'b0}}}	),
	.Re         (y1_re             			),
	.Im         (y1_im             			)
);

assign  y_re       =   x1_re + y1_re;
assign  y_im       =   x1_im + y1_im;


// dout = round(y) delay M = round(y(t-16))
trunc #(
	 .diw	(data_in_width+frac_data_out_width+1	)
	,.msb	(data_in_width+frac_data_out_width-1	)
	,.lsb	(frac_data_out_width			)
) round_u1 (clk, rstn, en, y_re, y_re_trunc);
trunc #(
	 .diw	(data_in_width+frac_data_out_width+1	)
	,.msb	(data_in_width+frac_data_out_width-1	)
	,.lsb	(frac_data_out_width			)
) round_u2 (clk, rstn, en, y_im, y_im_trunc);

assign  dout_re = y_re_trunc;
assign  dout_im = y_im_trunc;

endmodule
